@@ -4,7 +4,7 @@
  * Based on the `gen68360' board support package, and covered by the
  * original distribution terms.
  *
- *  $Id: bsp.h,v 1.15 2006/03/17 10:11:36 ralf Exp $
+ *  $Id$
  */
 
 /*  bsp.h
@@ -31,15 +31,6 @@ extern "C" {
 #include <rtems/iosupp.h>
 #include <rtems/clockdrv.h>
 
-/*
- *  confdefs.h overrides for this BSP:
- *   - termios serial ports (defaults to 1)
- *   - Interrupt stack space is not minimum if defined.
- */
-
-/* #define CONFIGURE_NUMBER_OF_TERMIOS_PORTS 2 */
-#define CONFIGURE_INTERRUPT_STACK_MEMORY  (4 * 1024)
-
 /*
  *  Simple spin delay in microsecond units for device drivers.
  *  This is very dependent on the clock speed of the target.
@@ -60,34 +51,10 @@ extern "C" {
 
 /* Structures */
 
-/*
- *  Device Driver Table Entries
- */
-
-/*
- * NOTE: Use the standard Console driver entry
- */
-
-/*
- * NOTE: Use the standard Clock driver entry
- */
-
-/*
- * How many libio files we want
- */
-
-#define BSP_LIBIO_MAX_FDS       20
-
-/* miscellaneous stuff assumed to exist */
-
-extern rtems_configuration_table BSP_Configuration;
-
 extern m68k_isr_entry M68Kvec[];   /* vector table address */
 
 /* functions */
 
-void bsp_cleanup( void );
-
 m68k_isr_entry set_vector(
   rtems_isr_entry     handler,
   rtems_vector_number vector,
