@@ -1,5 +1,5 @@
 /*
- *  $Id: leon_smc91111.c,v 1.5 2007/09/12 15:39:42 joel Exp $
+ *  $Id$
  */
 
 #include <rtems.h>
@@ -26,13 +26,13 @@
 
 
 #define SMC91111_BASE_ADDR (void*)0x20000300
-#define SMC91111_BASE_IRQ  LEON_TRAP_TYPE(4)
+#define SMC91111_BASE_IRQ  4
 #define SMC91111_BASE_PIO  4
 
 scmv91111_configuration_t leon_scmv91111_configuration = {
-  SMC91111_BASE_ADDR, /* base address */ 
-  SMC91111_BASE_IRQ,  /* vector number */ 
-  SMC91111_BASE_PIO,  /* PIO */ 
+  SMC91111_BASE_ADDR, /* base address */
+  SMC91111_BASE_IRQ,  /* vector number to IRQ funcs (on LEON IRQ is used) */
+  SMC91111_BASE_PIO,  /* PIO */
   100,                /* 100b */
   1,                  /* fulldx */
   1                   /* autoneg */
@@ -56,7 +56,7 @@ int rtems_smc91111_driver_attach_leon2(struct rtems_bsdnet_ifconfig *config)
   *((volatile unsigned int *)0x800000A8) |=
     (0xe0 | leon_scmv91111_configuration.pio)
       << (8 * ((leon_scmv91111_configuration.vector & 0x0f) - 4));
-  
+
   return _rtems_smc91111_driver_attach(config,&leon_scmv91111_configuration);
 
 };
