@@ -10,7 +10,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: amba.h,v 1.6 2007/09/06 00:01:30 joel Exp $
+ *  $Id$
  */
 
 #ifndef __AMBA_H__
@@ -35,7 +35,7 @@ extern "C" {
 #endif
 
 /* The AMBA Plug&Play info of the bus that the LEON3 sits on */
-extern amba_confarea_type amba_conf;
+extern struct ambapp_bus ambapp_plb;
 
 #ifdef __cplusplus
 }
