@@ -2,14 +2,14 @@
  *  Partition Manager
  *
  *
- *  COPYRIGHT (c) 1989-1999.
+ *  COPYRIGHT (c) 1989-2008.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: no-part.c,v 1.14 2007/05/09 18:44:20 joel Exp $
+ *  $Id$
  */
 
 #include <rtems/system.h>
@@ -20,8 +20,6 @@
 #include <rtems/score/thread.h>
 #include <rtems/score/interr.h>
 
-void _Partition_Manager_initialization(
-  uint32_t   maximum_partitions
-)
+void _Partition_Manager_initialization(void)
 {
 }
