@@ -6,7 +6,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: clr_flg.c,v 1.1 2007/04/17 20:32:12 joel Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -22,8 +22,8 @@
  */
 
 ER clr_flg(
-  ID   flgid,
-  UINT clrptn
+  ID   flgid __attribute__((unused)),
+  UINT clrptn __attribute__((unused))
 )
 {
   return E_OK;
