@@ -8,7 +8,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: read.c,v 1.12 2006/12/08 07:18:26 ralf Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -29,7 +29,7 @@ ssize_t read(
 
   rtems_libio_check_fd( fd );
   iop = rtems_libio_iop( fd );
-  rtems_libio_check_is_open(iop);
+  rtems_libio_check_is_open( iop );
   rtems_libio_check_buffer( buffer );
   rtems_libio_check_count( count );
   rtems_libio_check_permissions( iop, LIBIO_FLAGS_READ );
@@ -55,12 +55,12 @@ ssize_t read(
  *  This is the Newlib dependent reentrant version of read().
  */
 
-#if defined(RTEMS_NEWLIB)
+#if defined(RTEMS_NEWLIB) && !defined(HAVE__READ_R)
 
 #include <reent.h>
 
 ssize_t _read_r(
-  struct _reent *ptr,
+  struct _reent *ptr __attribute__((unused)),
   int            fd,
   void          *buf,
   size_t         nbytes
