@@ -1,5 +1,12 @@
 /*
- *  $Id: keygetspecific.c,v 1.6 2006/11/17 22:49:30 joel Exp $
+ *  COPYRIGHT (c) 1989-2007.
+ *  On-Line Applications Research Corporation (OAR).
+ *
+ *  The license and distribution terms for this file may be
+ *  found in the file LICENSE in this distribution or at
+ *  http://www.rtems.com/license/LICENSE.
+ *
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -33,16 +40,20 @@ void *pthread_getspecific(
 
   the_key = _POSIX_Keys_Get( key, &location );
   switch ( location ) {
-    case OBJECTS_ERROR:
-    case OBJECTS_REMOTE:   /* should never happen */
-      return NULL;
+
     case OBJECTS_LOCAL:
       api      = _Objects_Get_API( _Thread_Executing->Object.id );
       index    = _Objects_Get_index( _Thread_Executing->Object.id );
       key_data = (void *) the_key->Values[ api ][ index ];
       _Thread_Enable_dispatch();
       return key_data;
+
+#if defined(RTEMS_MULTIPROCESSING)
+    case OBJECTS_REMOTE:   /* should never happen */
+#endif
+    case OBJECTS_ERROR:
+      break;
   }
-  (void) POSIX_BOTTOM_REACHED();
-  return (void *)NULL;
+
+  return NULL;
 }
