@@ -8,7 +8,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: dpmemident.c,v 1.7 2005/01/18 09:03:43 ralf Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -42,15 +42,15 @@
  */
 
 rtems_status_code rtems_port_ident(
-  rtems_name    name,
-  Objects_Id   *id
+  rtems_name  name,
+  rtems_id   *id
 )
 {
   Objects_Name_or_id_lookup_errors  status;
 
-  status = _Objects_Name_to_id(
+  status = _Objects_Name_to_id_u32(
     &_Dual_ported_memory_Information,
-    (Objects_Name) name,
+    name,
     OBJECTS_SEARCH_ALL_NODES,
     id
   );
