@@ -8,7 +8,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: iounregisterdriver.c,v 1.1 2007/05/29 19:56:35 joel Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -17,6 +17,8 @@
 
 #include <rtems/system.h>
 #include <rtems/io.h>
+#include <rtems/rtems/intr.h>
+#include <rtems/score/thread.h>
 #include <string.h>
 
 /*
@@ -36,13 +38,20 @@ rtems_status_code rtems_io_unregister_driver(
   rtems_device_major_number major
 )
 {
+  if ( rtems_interrupt_is_in_progress() )
+    return RTEMS_CALLED_FROM_ISR;
+
   if ( major < _IO_Number_of_drivers ) {
+    _Thread_Disable_dispatch();
     memset(
       &_IO_Driver_address_table[major],
       0,
       sizeof( rtems_driver_address_table )
     );
+    _Thread_Enable_dispatch();
+
     return RTEMS_SUCCESSFUL;
   }
+
   return RTEMS_UNSATISFIED;
 }
