@@ -13,7 +13,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: cpu.h,v 1.24 2007/07/31 16:48:38 ralf Exp $
+ *  $Id$
  */
 
 #ifndef _RTEMS_SCORE_CPU_H
@@ -25,7 +25,11 @@ extern "C" {
 
 #include <rtems/score/m68k.h>              /* pick up machine definitions */
 #ifndef ASM
-#include <rtems/score/types.h>
+  #include <rtems/score/types.h>
+#else
+  /* FIXME */
+  #define TRUE 1
+  #define FALSE 0
 #endif
 
 /* conditional compilation parameters */
@@ -33,6 +37,19 @@ extern "C" {
 #define CPU_INLINE_ENABLE_DISPATCH       TRUE
 #define CPU_UNROLL_ENQUEUE_PRIORITY      FALSE
 
+/*
+ *  Does the CPU follow the simple vectored interrupt model?
+ *
+ *  If TRUE, then RTEMS allocates the vector table it internally manages.
+ *  If FALSE, then the BSP is assumed to allocate and manage the vector
+ *  table
+ *
+ *  M68K Specific Information:
+ *
+ *  XXX document implementation including references if appropriate
+ */
+#define CPU_SIMPLE_VECTORED_INTERRUPTS TRUE
+
 /*
  *  Use the m68k's hardware interrupt stack support and have the
  *  interrupt manager allocate the memory for it.
@@ -49,7 +66,7 @@ extern "C" {
 
 /*
  *  Does the RTEMS invoke the user's ISR with the vector number and
- *  a pointer to the saved interrupt frame (1) or just the vector 
+ *  a pointer to the saved interrupt frame (1) or just the vector
  *  number (0)?
  */
 
@@ -64,16 +81,16 @@ extern "C" {
  *         must be contexted switched on a per task basis.
  */
 
-#if ( M68K_HAS_FPU == 1 )
-#define CPU_HARDWARE_FP     TRUE
-#define CPU_SOFTWARE_FP     FALSE
+#if ( M68K_HAS_FPU == 1 ) || ( M68K_HAS_EMAC == 1 )
+  #define CPU_HARDWARE_FP TRUE
+  #define CPU_SOFTWARE_FP FALSE
 #else
-#define CPU_HARDWARE_FP     FALSE
-#if defined(__GNUC__)
-#define CPU_SOFTWARE_FP     TRUE
-#else
-#define CPU_SOFTWARE_FP     FALSE
-#endif
+  #define CPU_HARDWARE_FP FALSE
+  #if defined( __GNUC__ )
+    #define CPU_SOFTWARE_FP TRUE
+  #else
+    #define CPU_SOFTWARE_FP FALSE
+  #endif
 #endif
 
 /*
@@ -89,7 +106,7 @@ extern "C" {
 
 #define CPU_PROVIDES_IDLE_THREAD_BODY    TRUE
 #define CPU_STACK_GROWS_UP               FALSE
-#define CPU_STRUCTURE_ALIGNMENT
+#define CPU_STRUCTURE_ALIGNMENT          __attribute__ ((aligned (4)))
 
 /*
  *  Define what is required to specify how the network to host conversion
@@ -99,7 +116,16 @@ extern "C" {
 #define CPU_BIG_ENDIAN                           TRUE
 #define CPU_LITTLE_ENDIAN                        FALSE
 
+#if ( CPU_HARDWARE_FP == TRUE ) && !defined( __mcoldfire__ )
+  #if defined( __mc68060__ )
+    #define M68K_FP_STATE_SIZE 16
+  #else
+    #define M68K_FP_STATE_SIZE 216
+  #endif
+#endif
+
 #ifndef ASM
+
 /* structures */
 
 /*
@@ -120,50 +146,145 @@ typedef struct {
   void       *a5;                /* (a5) address register 5 */
   void       *a6;                /* (a6) address register 6 */
   void       *a7_msp;            /* (a7) master stack pointer */
-}   Context_Control;
+  #if defined( __mcoldfire__ ) && ( M68K_HAS_FPU == 1 )
+    uint8_t   fpu_dis;
+  #endif
+} Context_Control;
+
+#define _CPU_Context_Get_SP( _context ) \
+  (_context)->a7_msp
 
 /*
- *  Floating point context ares
+ *  Floating point context areas and support routines
  */
 
-#if (CPU_SOFTWARE_FP == TRUE)
+#if ( CPU_SOFTWARE_FP == TRUE )
+  /*
+   *  This is the same as gcc's view of the software FP condition code
+   *  register _fpCCR.  The implementation of the emulation code is
+   *  in the gcc-VERSION/config/m68k directory.  This structure is
+   *  correct as of gcc 2.7.2.2.
+   */
+  typedef struct {
+    uint16_t _exception_bits;
+    uint16_t _trap_enable_bits;
+    uint16_t _sticky_bits;
+    uint16_t _rounding_mode;
+    uint16_t _format;
+    uint16_t _last_operation;
+    union {
+      float sf;
+      double df;
+    } _operand1;
+    union {
+      float sf;
+      double df;
+    } _operand2;
+  } Context_Control_fp;
 
-/*
- *  This is the same as gcc's view of the software FP condition code
- *  register _fpCCR.  The implementation of the emulation code is
- *  in the gcc-VERSION/config/m68k directory.  This structure is
- *  correct as of gcc 2.7.2.2.
- */
+  /*
+   *  This software FP implementation is only for GCC.
+   */
+  #define _CPU_Context_Fp_start( _base, _offset ) \
+     ((void *) _Addresses_Add_offset( (_base), (_offset) ) )
 
-typedef struct {
-  uint16_t     _exception_bits;
-  uint16_t     _trap_enable_bits;
-  uint16_t     _sticky_bits;
-  uint16_t     _rounding_mode;
-  uint16_t     _format;
-  uint16_t     _last_operation;
-  union {
-    float sf;
-    double df;
-  } _operand1;
-  union {
-    float sf;
-    double df;
-  } _operand2;
-} Context_Control_fp;
+  #define _CPU_Context_Initialize_fp( _fp_area ) \
+     { \
+       Context_Control_fp *_fp; \
+       _fp = *(Context_Control_fp **)_fp_area; \
+       _fp->_exception_bits = 0; \
+       _fp->_trap_enable_bits = 0; \
+       _fp->_sticky_bits = 0; \
+       _fp->_rounding_mode = 0;  /* ROUND_TO_NEAREST */ \
+       _fp->_format = 0;         /* NIL */ \
+       _fp->_last_operation = 0; /* NOOP */ \
+       _fp->_operand1.df = 0; \
+       _fp->_operand2.df = 0; \
+     }
+#endif
 
-#else 
+#if ( CPU_HARDWARE_FP == TRUE )
+  #if defined( __mcoldfire__ )
+    /* We need memset() to initialize the FP context */
+    #include <string.h>
 
-/*
- *  FP context save area for the M68881/M68882 numeric coprocessors.
- */
+    #if ( M68K_HAS_FPU == 1 )
+      /*
+       * The Cache Control Register (CACR) has write-only access.  It is also
+       * used to enable and disable the FPU.  We need to maintain a copy of
+       * this register to allow per thread values.
+       */
+      extern uint32_t _CPU_cacr_shadow;
+    #endif
 
-typedef struct {
-  uint8_t     fp_save_area[332];    /*   216 bytes for FSAVE/FRESTORE    */
-                                    /*    96 bytes for FMOVEM FP0-7      */
-                                    /*    12 bytes for FMOVEM CREGS      */
-                                    /*     4 bytes for non-null flag     */
-} Context_Control_fp;
+    /* We assume that each ColdFire core with a FPU has also an EMAC unit */
+    typedef struct {
+      uint32_t emac_macsr;
+      uint32_t emac_acc0;
+      uint32_t emac_acc1;
+      uint32_t emac_acc2;
+      uint32_t emac_acc3;
+      uint32_t emac_accext01;
+      uint32_t emac_accext23;
+      uint32_t emac_mask;
+      #if ( M68K_HAS_FPU == 1 )
+        uint16_t fp_state_format;
+        uint16_t fp_state_fpcr;
+        double fp_state_op;
+        uint32_t fp_state_fpsr;
+
+        /*
+         * We need to save the FP Instruction Address Register (FPIAR), because
+         * a context switch can occur within a FP exception before the handler
+         * was able to save this register.
+         */
+        uint32_t fp_fpiar;
+
+        double fp_data [8];
+      #endif
+    } Context_Control_fp;
+
+    #define _CPU_Context_Fp_start( _base, _offset ) \
+      ((void *) _Addresses_Add_offset( (_base), (_offset) ))
+
+    /*
+     * The reset value for all context relevant registers except the FP data
+     * registers is zero.  The reset value of the FP data register is NAN.  The
+     * restore of the reset FP state will reset the FP data registers, so the
+     * initial value of them can be arbitrary here.
+     */
+    #define _CPU_Context_Initialize_fp( _fp_area ) \
+      memset( *(_fp_area), 0, sizeof( Context_Control_fp ) )
+  #else
+    /*
+     *  FP context save area for the M68881/M68882 and 68060 numeric coprocessors.
+     */
+
+    typedef struct {
+      /*
+       * M68K_FP_STATE_SIZE bytes for FSAVE/FRESTORE
+       * 96 bytes for FMOVEM FP0-7
+       * 12 bytes for FMOVEM CREGS
+       * 4 bytes for non-null flag
+       */
+      uint8_t fp_save_area [M68K_FP_STATE_SIZE + 112];
+    } Context_Control_fp;
+
+    #define _CPU_Context_Fp_start( _base, _offset ) \
+       ( \
+         (void *) _Addresses_Add_offset( \
+            (_base), \
+            (_offset) + CPU_CONTEXT_FP_SIZE - 4 \
+         ) \
+       )
+
+    #define _CPU_Context_Initialize_fp( _fp_area ) \
+       { \
+         uint32_t   *_fp_context = (uint32_t *)*(_fp_area); \
+         *(--(_fp_context)) = 0; \
+         *(_fp_area) = (void *)(_fp_context); \
+       }
+  #endif
 #endif
 
 /*
@@ -185,45 +306,12 @@ typedef struct {
   uint32_t   a0, a1, a2, a3, a4, a5, a6, a7;
 } CPU_Exception_frame;
 
-/*
- *  The following table contains the information required to configure
- *  the m68k specific parameters.
- */
-
-typedef struct {
-  void       (*pretasking_hook)( void );
-  void       (*predriver_hook)( void );
-  void       (*postdriver_hook)( void );
-  void       (*idle_task)( void );
-  boolean      do_zero_of_workspace;
-  uint32_t     idle_task_stack_size;
-  uint32_t     interrupt_stack_size;
-  uint32_t     extra_mpci_receive_server_stack;
-  void *     (*stack_allocate_hook)( uint32_t   );
-  void       (*stack_free_hook)( void* );
-  /* end of fields required on all CPUs */
-
-  m68k_isr    *interrupt_vector_table;
-}   rtems_cpu_table;
-
-/*
- *  Macros to access required entires in the CPU Table are in 
- *  the file rtems/system.h.
- */
-
-/*
- *  Macros to access M68K specific additions to the CPU Table
- */
-
-#define rtems_cpu_configuration_get_interrupt_vector_table() \
-   (_CPU_Table.interrupt_vector_table)
-
 /* variables */
 
-SCORE_EXTERN void                   *_CPU_Interrupt_stack_low;
-SCORE_EXTERN void                   *_CPU_Interrupt_stack_high;
+SCORE_EXTERN void               *_CPU_Interrupt_stack_low;
+SCORE_EXTERN void               *_CPU_Interrupt_stack_high;
 
-extern void*                     _VBR; 
+extern void*                     _VBR;
 
 #if ( M68K_HAS_VBR == 0 )
 
@@ -248,9 +336,10 @@ typedef struct {
 #define M68K_JMP     0x4EF9
 
       /* points to jsr-exception-table in targets wo/ VBR register */
-SCORE_EXTERN _CPU_ISR_handler_entry _CPU_ISR_jump_table[256]; 
+SCORE_EXTERN _CPU_ISR_handler_entry _CPU_ISR_jump_table[256];
 
 #endif /* M68K_HAS_VBR */
+
 #endif /* ASM */
 
 /* constants */
@@ -293,7 +382,12 @@ SCORE_EXTERN _CPU_ISR_handler_entry _CPU_ISR_jump_table[256];
  *  Minimum size of a thread's stack.
  */
 
-#define CPU_STACK_MINIMUM_SIZE           4096
+#define CPU_STACK_MINIMUM_SIZE           M68K_CPU_STACK_MINIMUM_SIZE
+
+/*
+ *  Maximum priority of a thread. Note based from 0 which is the idle task.
+ */
+#define CPU_PRIORITY_MAXIMUM             M68K_CPU_PRIORITY_MAXIMUM
 
 /*
  *  m68k is pretty tolerant of alignment.  Just put things on 4 byte boundaries.
@@ -353,67 +447,29 @@ uint32_t   _CPU_ISR_Get_level( void );
  *     + initialize an FP context area
  */
 
+#if (defined(__mcoldfire__) && ( M68K_HAS_FPU == 1 ))
 #define _CPU_Context_Initialize( _the_context, _stack_base, _size, \
                                  _isr, _entry_point, _is_fp ) \
    do { \
      uint32_t   _stack; \
      \
      (_the_context)->sr      = 0x3000 | ((_isr) << 8); \
-     _stack                  = (uint32_t  )(_stack_base) + (_size) - 4; \
+     _stack                  = (uint32_t)(_stack_base) + (_size) - 4; \
      (_the_context)->a7_msp  = (void *)_stack; \
      *(void **)_stack        = (void *)(_entry_point); \
+     (_the_context)->fpu_dis = (_is_fp == TRUE) ? 0x00 : 0x10;          \
    } while ( 0 )
-
-#define _CPU_Context_Restart_self( _the_context ) \
-  { asm volatile( "movew %0,%%sr ; " \
-                  "moval %1,%%a7 ; " \
-                  "rts"  \
-        : "=d" ((_the_context)->sr), "=d" ((_the_context)->a7_msp) \
-        : "0" ((_the_context)->sr), "1" ((_the_context)->a7_msp) ); \
-  }
-
-/*
- *  Floating Point Context Area Support routines
- */
-
-#if (CPU_SOFTWARE_FP == TRUE)
-
-/*
- *  This software FP implementation is only for GCC.
- */
-
-#define _CPU_Context_Fp_start( _base, _offset ) \
-   ((void *) _Addresses_Add_offset( (_base), (_offset) ) )
-
-
-#define _CPU_Context_Initialize_fp( _fp_area ) \
-   { \
-   Context_Control_fp *_fp; \
-   _fp = *(Context_Control_fp **)_fp_area; \
-   _fp->_exception_bits = 0; \
-   _fp->_trap_enable_bits = 0; \
-   _fp->_sticky_bits = 0; \
-   _fp->_rounding_mode = 0;  /* ROUND_TO_NEAREST */ \
-   _fp->_format = 0;         /* NIL */ \
-   _fp->_last_operation = 0;  /* NOOP */ \
-   _fp->_operand1.df = 0; \
-   _fp->_operand2.df = 0; \
-   }
 #else
-#define _CPU_Context_Fp_start( _base, _offset ) \
-   ((void *) \
-     _Addresses_Add_offset( \
-        (_base), \
-        (_offset) + CPU_CONTEXT_FP_SIZE - 4 \
-     ) \
-   )
-
-#define _CPU_Context_Initialize_fp( _fp_area ) \
-   { uint32_t   *_fp_context = (uint32_t   *)*(_fp_area); \
-     \
-     *(--(_fp_context)) = 0; \
-     *(_fp_area) = (uint8_t   *)(_fp_context); \
-   }
+#define _CPU_Context_Initialize( _the_context, _stack_base, _size,      \
+                                 _isr, _entry_point, _is_fp )           \
+   do {                                                                 \
+     uint32_t   _stack;                                                 \
+                                                                        \
+     (_the_context)->sr      = 0x3000 | ((_isr) << 8);                  \
+     _stack                  = (uint32_t  )(_stack_base) + (_size) - 4; \
+     (_the_context)->a7_msp  = (void *)_stack;                          \
+     *(void **)_stack        = (void *)(_entry_point);                  \
+   } while ( 0 )
 #endif
 
 /* end of Context handler macros */
@@ -427,7 +483,7 @@ uint32_t   _CPU_ISR_Get_level( void );
  *         is TRUE.
  */
 
-void _CPU_Thread_Idle_body( void );
+void *_CPU_Thread_Idle_body( uintptr_t ignored );
 
 /*
  *  Fatal Error manager macros
@@ -475,12 +531,19 @@ void _CPU_Thread_Idle_body( void );
 #define CPU_USE_GENERIC_BITFIELD_CODE FALSE
 #define CPU_USE_GENERIC_BITFIELD_DATA FALSE
 
+#if ( M68K_HAS_BFFFO != 1 )
+/*
+ *  Lookup table for BFFFO simulation
+ */
+extern const unsigned char _CPU_m68k_BFFFO_table[256];
+#endif
+
 #if ( M68K_HAS_BFFFO == 1 )
 
 #define _CPU_Bitfield_Find_first_bit( _value, _output ) \
   asm volatile( "bfffo (%1),#0,#16,%0" : "=d" (_output) : "a" (&_value));
 
-#elif ( M68K_HAS_ISA_APLUS == 1 )
+#elif ( __mcfisaaplus__ )
   /* This is simplified by the fact that RTEMS never calls it with _value=0 */
 #define _CPU_Bitfield_Find_first_bit( _value, _output ) \
     asm volatile ( \
@@ -495,9 +558,9 @@ void _CPU_Thread_Idle_body( void );
    _CPU_Priority_bits_index is not needed), handles the 0 case, and
    does not molest _value -- jsg */
 #if ( defined(__mcoldfire__) )
+
 #define _CPU_Bitfield_Find_first_bit( _value, _output ) \
   { \
-    extern const unsigned char __BFFFOtable[256]; \
     register int dumby; \
     \
     asm volatile ( \
@@ -512,13 +575,12 @@ void _CPU_Thread_Idle_body( void );
        "   addq.l  #8,%0\n"      \
        "0: and.l   #0xff,%0\n"   \
        : "=&d" ((_output)), "=&d" ((dumby))    \
-       : "d" ((_value)), "ao" ((__BFFFOtable)) \
+       : "d" ((_value)), "ao" ((_CPU_m68k_BFFFO_table)) \
        : "cc" ) ; \
   }
 #elif ( M68K_HAS_EXTB_L == 1 )
 #define _CPU_Bitfield_Find_first_bit( _value, _output ) \
   { \
-    extern const unsigned char __BFFFOtable[256]; \
     register int dumby; \
     \
     asm volatile ( "   move.w  %2,%1\n"        \
@@ -531,13 +593,12 @@ void _CPU_Thread_Idle_body( void );
        "   add.b   (%3,%2.w),%0\n" \
        "0:\n"                      \
        : "=&d" ((_output)), "=&d" ((dumby)) \
-       : "d" ((_value)), "ao" ((__BFFFOtable)) \
+       : "d" ((_value)), "ao" ((_CPU_m68k_BFFFO_table)) \
        : "cc" ) ; \
   }
 #else
 #define _CPU_Bitfield_Find_first_bit( _value, _output ) \
   { \
-    extern const unsigned char __BFFFOtable[256]; \
     register int dumby; \
     \
     asm volatile ( "   move.w  %2,%1\n"        \
@@ -550,7 +611,7 @@ void _CPU_Thread_Idle_body( void );
        "   add.b   (%3,%2.w),%0\n" \
        "0:\n"                      \
        : "=&d" ((_output)), "=&d" ((dumby)) \
-       : "d" ((_value)), "ao" ((__BFFFOtable)) \
+       : "d" ((_value)), "ao" ((_CPU_m68k_BFFFO_table)) \
        : "cc" ) ; \
   }
 #endif
@@ -585,18 +646,15 @@ void _CPU_Thread_Idle_body( void );
  *  This routine performs CPU dependent initialization.
  */
 
-void _CPU_Initialize(
-  rtems_cpu_table  *cpu_table,
-  void      (*thread_dispatch)
-);
+void _CPU_Initialize(void);
 
 /*
  *  _CPU_ISR_install_raw_handler
  *
- *  This routine installs a "raw" interrupt handler directly into the 
+ *  This routine installs a "raw" interrupt handler directly into the
  *  processor's vector table.
  */
- 
+
 void _CPU_ISR_install_raw_handler(
   uint32_t    vector,
   proc_ptr    new_handler,
@@ -634,6 +692,10 @@ void _CPU_Context_switch(
   Context_Control  *heir
 );
 
+void _CPU_Context_Restart_self(
+  Context_Control  *the_context
+);
+
 /*
  *  _CPU_Context_save_fp
  *
@@ -658,7 +720,7 @@ void _CPU_Context_restore_fp(
 /*
  *  Hooks for the Floating Point Support Package (FPSP) provided by Motorola
  *
- *  NOTES:  
+ *  NOTES:
  *
  *  Motorola 68k family CPU's before the 68040 used a coprocessor
  *  (68881 or 68882) to handle floating point.  The 68040 has internal
