@@ -2,14 +2,14 @@
  *  Thread Queue Handler
  *
  *
- *  COPYRIGHT (c) 1989-1999.
+ *  COPYRIGHT (c) 1989-2008.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: threadqfirst.c,v 1.5 2005/01/27 05:57:05 ralf Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -43,19 +43,12 @@ Thread_Control *_Thread_queue_First(
   Thread_queue_Control *the_thread_queue
 )
 {
-  Thread_Control *the_thread;
+  Thread_Control * (*first_p)(Thread_queue_Control *);
 
-  switch ( the_thread_queue->discipline ) {
-    case THREAD_QUEUE_DISCIPLINE_FIFO:
-      the_thread = _Thread_queue_First_fifo( the_thread_queue );
-      break;
-    case THREAD_QUEUE_DISCIPLINE_PRIORITY:
-      the_thread = _Thread_queue_First_priority( the_thread_queue );
-      break;
-    default:              /* this is only to prevent warnings */
-      the_thread = NULL;
-      break;
-  }
+  if ( the_thread_queue->discipline == THREAD_QUEUE_DISCIPLINE_PRIORITY )
+      first_p = _Thread_queue_First_priority;
+  else /* must be THREAD_QUEUE_DISCIPLINE_FIFO */
+      first_p = _Thread_queue_First_fifo;
 
-  return the_thread;
+  return (*first_p)( the_thread_queue );
 }
