@@ -1,4 +1,4 @@
-/** 
+/**
  *  @file  score/src/timespecisvalid.c
  */
 
@@ -10,7 +10,7 @@
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: timespecisvalid.c,v 1.1 2007/04/05 21:17:27 joel Exp $
+ *  $Id$
  */
 
 #if HAVE_CONFIG_H
@@ -23,21 +23,21 @@
 #include <rtems/score/timespec.h>
 #include <rtems/score/tod.h>
 
-boolean _Timespec_Is_valid(
+bool _Timespec_Is_valid(
   const struct timespec *time
 )
 {
   if ( !time )
-    return FALSE;
+    return false;
 
   if ( time->tv_sec < 0 )
-    return FALSE;
+    return false;
 
   if ( time->tv_nsec < 0 )
-    return FALSE;
+    return false;
 
   if ( time->tv_nsec >= TOD_NANOSECONDS_PER_SECOND )
-    return FALSE;
+    return false;
 
-  return TRUE;
+  return true;
 }
