@@ -2,18 +2,17 @@
  *  This test exercises stat() via fstat() and generates as many of the
  *  path evaluation cases as possible.
  *
- *  COPYRIGHT (c) 1989-1999.
+ *  COPYRIGHT (c) 1989-2009.
  *  On-Line Applications Research Corporation (OAR).
  *
  *  The license and distribution terms for this file may be
  *  found in the file LICENSE in this distribution or at
  *  http://www.rtems.com/license/LICENSE.
  *
- *  $Id: test.c,v 1.17 2004/04/16 09:23:26 ralf Exp $
+ *  $Id$
  */
 
 #include <tmacros.h>
-#include <assert.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <limits.h>
@@ -27,7 +26,7 @@
 #include <pmacros.h>
 
 #define MAXSYMLINK 5   /* There needs to be a better way of getting this. */
-#define TIMEOUT_VALUE  ( 5 * TICKS_PER_SECOND )
+#define TIMEOUT_VALUE  ( 5 * rtems_clock_get_ticks_per_second() )
 
 
 /*
@@ -180,7 +179,7 @@ void stat_a_file(
   int         minor2;
 
 
-  assert( file );
+  rtems_test_assert( file );
 
   printf( "stat( %s ) returned ", file );
   fflush( stdout );
@@ -196,21 +195,18 @@ void stat_a_file(
 
 
     printf("\n...st_dev     (0x%x:0x%x)\n", major1, minor1 );
-    printf(  "...st_ino     %x\n", (int) statbuf.st_ino );
+    printf(  "...st_ino     %" PRIxino_t "\n", statbuf.st_ino );
     printf(  "...st_mode    %o\n", (unsigned int) statbuf.st_mode );
     printf(  "...st_nlink   %x\n", statbuf.st_nlink );
     printf(  "...st_uid     %d\n", statbuf.st_uid );
     printf(  "...st_gid     %d\n", statbuf.st_gid );
     printf(  "...st_rdev    (0x%x:0x%x)\n", major2, minor2 );
-    printf(  "...st_size    %d\n",(unsigned int) statbuf.st_size );
+    printf(  "...st_size    %" PRIdoff_t "\n", statbuf.st_size );
     printf(  "...st_atime   %s", ctime( &statbuf.st_atime ) );
     printf(  "...st_mtime   %s", ctime( &statbuf.st_mtime ) );
     printf(  "...st_ctime   %s", ctime( &statbuf.st_ctime ) );
-#if defined(__svr4__) && !defined(__PPC__) && !defined(__sun__)
-    printf(  "...st_blksize %x\n", statbuf.st_blksize );
-    printf(  "...st_blocks  %x\n", statbuf.st_blocks );
-#endif
-
+    printf(  "...st_blksize %" PRIxblksize_t "\n", statbuf.st_blksize );
+    printf(  "...st_blocks  %" PRIxblkcnt_t "\n", statbuf.st_blocks );
   }
 }
 
@@ -276,7 +272,7 @@ void make_multiple_files(
   while ( files[i] ) {
     printf( "Making file %s\n", files[i] );
     status = mknod( files[i], ( S_IFREG | S_IROTH|S_IWOTH ), 0LL );
-    assert( !status );
+    rtems_test_assert( !status );
     i++;
   }
   puts( "" );
@@ -293,7 +289,7 @@ void make_multiple_bad_files(
   while ( files[i] ) {
     printf( "Making file %s ", files[i] );
     status = mknod( files[i], ( S_IFREG | S_IROTH|S_IWOTH ), 0LL );
-    assert( status );
+    rtems_test_assert( status );
     printf( ": %s\n", strerror( errno ) );
     i++;
   }
@@ -312,27 +308,27 @@ void make_multiple_links(
   while ( new[i] && existing[i] ) {
     printf( "Making file %s\n", new[i] );
     status = link( existing[i], new[i] );
-    assert( !status );
+    rtems_test_assert( !status );
     i++;
   }
   puts( "" );
 
   status = link( "fred", "bob" );
-  assert( status == -1 );
+  rtems_test_assert( status == -1 );
 
   status = link( existing[1], "doug/bob" );
-  assert( status == -1 );
+  rtems_test_assert( status == -1 );
 }
 
 
-void make_too_many_links()
+void make_too_many_links(void)
 {
   int    i;
   int    status;
   char   name [20];
 
   status = mkdir("/dummy", S_IRWXU );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   for (i=1; i<= LINK_MAX; i++) {
 
@@ -340,9 +336,9 @@ void make_too_many_links()
     printf( "Making file %s\n", name );
     status = link("/dummy" , name );
     if( i < LINK_MAX )
-       assert( !status );
+       rtems_test_assert( !status );
     else
-       assert( status == -1 );
+       rtems_test_assert( status == -1 );
 
   }
 }
@@ -361,25 +357,25 @@ void make_a_symlink(
 
   printf( "Making file %s\n", new );
   status = symlink( existing, new );
-  assert( !status );
+  rtems_test_assert( !status );
 
   printf( "Verify with readlink\n");
   status = readlink( new, buf, 100 );
   len = strlen( existing );
-  assert ( status == len );
+  rtems_test_assert ( status == len );
 
   status = readlink( new, buf, 3 );
   len = strlen( existing );
   if (len < 3 )
-    assert( status == len );
+    rtems_test_assert( status == len );
   else
-    assert( status == 3 );
+    rtems_test_assert( status == 3 );
 
   status = strcmp( existing, buf );
-  assert( !status );
+  rtems_test_assert( !status );
 }
 
-void make_multiple_symlinks()
+void make_multiple_symlinks(void)
 {
  int  status;
 
@@ -397,7 +393,7 @@ void make_multiple_symlinks()
  stat_a_file( SymLinks[4] );
 
  status = symlink(  "//links", "bob/frank" );
- assert (status == -1);
+ rtems_test_assert (status == -1);
 
 }
 /*
@@ -410,9 +406,9 @@ void make_too_many_symlinks()
     sprintf( name1, "SymLink%d", i );
     status = symlink( "/dummy", name1 );
     if( i < MAXSYMLINK )
-       assert( !status );
+       rtems_test_assert( !status );
     else
-       assert( status == -1 );
+       rtems_test_assert( status == -1 );
   }
 }
 */
@@ -456,7 +452,7 @@ void make_multiple_directories(
   while ( files[i] ) {
     printf( "Making directory %s\n", files[i] );
     status = mkdir( files[i], S_IRWXU );
-    assert( !status );
+    rtems_test_assert( !status );
     i++;
   }
   puts( "" );
@@ -467,33 +463,34 @@ void make_multiple_directories(
  */
 
 
-void Cause_faults()
+void Cause_faults(void)
 {
   int                                   fd;
   int                                   status;
   char                                  longer_name[100];
-  rtems_filesystem_mount_table_entry_t *mt_entry;
 
   /*
    * Verify chmod with an invalid type.
    */
 
+#if 0
   printf("\n\nPass an invalid mode to chmod should fail with EPERM \n" );
   status = chmod( Files[0], S_IFREG );
-  assert( status == -1 );
-  assert( errno == EPERM );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == EPERM );
+#endif
 
   /*
    * Change file to executable then try to chdir to it.
    */
 
   status = chmod( Files[0], S_IXUSR );
-  assert( status != -1 );
+  rtems_test_assert( status != -1 );
 
   printf("chdir to a file should fail with ENOTDIR\n");
   status = chdir( Files[0] );
-  assert( status == -1 );
-  assert( errno == ENOTDIR );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == ENOTDIR );
 
   /*
    * Change mode to read/write on a directory.
@@ -502,28 +499,28 @@ void Cause_faults()
 
   printf("Verify RWX permission on %s via access\n", Directories[0]);
   status = access( Directories[0], ( R_OK | W_OK | X_OK )  );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   printf( "chmod of %s to Read/Write\n", Directories[0] );
   status = chmod( Directories[0], (S_IXGRP | S_IXOTH) );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   printf( "chmod fred should fail with ENOENT\n" );
   status = chmod( "fred", (S_IXGRP | S_IXOTH) );
-  assert( status == -1 );
-  assert( errno == ENOENT );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == ENOENT );
 
   strcpy(longer_name, Directories[0] );
   strcat(longer_name, "/BADNAME" );
   printf( "Create under %s should fail with EACCES\n", Directories[0] );
   status = mkdir( longer_name , S_IRWXU );
-  assert( status == -1 );
-  assert( errno == EACCES );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == EACCES );
 
   printf("chdir to %s should fail with EACCES\n", Directories[4] );
   status = chdir( Directories[4] );
-  assert( status == -1 );
-  assert( errno == EACCES );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == EACCES );
 
   /*
    * Check stat with a NULL buffer.
@@ -531,8 +528,8 @@ void Cause_faults()
 
   printf("Stat with a NULL buffer should fail with EFAULT\n");
   status = stat( Directories[0], NULL );
-  assert( status == -1 );
-  assert( errno == EFAULT );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == EFAULT );
 
   /*
    * Set current to a directory with no owner permissions.
@@ -541,33 +538,33 @@ void Cause_faults()
 
   printf( "\n\nchmod of %s to Read/Write\n", Directories[0] );
   status = chmod( Directories[0], (S_IXGRP | S_IXOTH) );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   printf("mkdir %s should fail with EACCESS\n", longer_name );
   status = mkdir( longer_name , S_IRWXU );
-  assert( status == -1 );
-  assert( errno == EACCES );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == EACCES );
 
   printf("\n%s Should exist ( access )\n",Directories[0] );
   status = access( Directories[0], F_OK );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
   printf("\n%s Should have read  permission( access )\n",Directories[0] );
   status = access( Directories[0], R_OK );
-  assert( status != 0 );
+  rtems_test_assert( status != 0 );
   printf("\n%s Should have write permission( access )\n",Directories[0] );
   status = access( Directories[0], W_OK );
-  assert( status != 0 );
+  rtems_test_assert( status != 0 );
   printf("\n%s Should not have execute permission( access )\n",Directories[0] );
   status = access( Directories[0], X_OK );
-  assert( status != 0 );
+  rtems_test_assert( status != 0 );
 
   printf("\nRestore %s to RWX\n",Directories[0] );
   status = chmod( Directories[0], S_IRWXU );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   printf("chdir to /my_mount_point \n");
   status = chdir( "/my_mount_point" );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   /*
    * Remove one of the directories.
@@ -576,21 +573,21 @@ void Cause_faults()
 
   printf( "Remove %s\n", Directories[5] );
   status = rmdir( Directories[5] );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   stat_a_file( Directories[5] );
   status = access( Directories[5], F_OK );
-  assert( status != 0 );
+  rtems_test_assert( status != 0 );
 
   stat_a_file( Links_to_Dirs[5] );
   status = readlink( Links_to_Dirs[5], longer_name, 3 );
-  assert( status == -1 );
-  assert( errno == EINVAL );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == EINVAL );
 
   stat_a_file( Links_to_dirlinks[5] );
   printf("Chdir to %s\n", Links_to_Dirs[5] );
   status = chdir( Links_to_Dirs[5] );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   /*
    * Verify we cannot move up from a node with no parent node.
@@ -598,8 +595,8 @@ void Cause_faults()
 
   printf("Chdir to .. should fail with ENOENT\n" );
   status = chdir( ".." );
-  assert( status == -1 );
-  assert( errno == ENOENT );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == ENOENT );
 
   /*
    * Create a subdirectory under the dangling node.
@@ -607,16 +604,16 @@ void Cause_faults()
 
   printf("mkdir ../t should fail with ENOENT\n" );
   status = mkdir( "../t" , S_IRWXU );
-  assert( status == -1 );
-  assert( errno == ENOENT );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == ENOENT );
 
   printf("mkdir t\n");
   status = mkdir( "t" , S_IRWXU );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   printf("chdir to /my_mount_point\n");
   status = chdir( "/my_mount_point" );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   /*
    * Check rmdir, rmnod, and unlink
@@ -624,49 +621,49 @@ void Cause_faults()
 
   printf("rmdir %s should fail with ENOTDIR\n", Links_to_Dirs[5] );
   status = rmdir( Links_to_Dirs[5] );
-  assert( status == -1 );
-  assert( errno == ENOTDIR );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == ENOTDIR );
 
   printf("unlink %s\n", Links_to_Dirs[5] );
   status = unlink( Links_to_Dirs[5] );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   printf("unlink %s should fail with ENOTEMPTY\n", Links_to_dirlinks[5] );
   status = unlink(  Links_to_dirlinks[5] );
-  assert( status == -1 );
-  assert( errno == ENOTEMPTY );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == ENOTEMPTY );
 
   strcpy( longer_name,  Links_to_dirlinks[5] );
   strcat( longer_name, "/t");
   printf("rmdir %s\n", longer_name );
   status = rmdir( longer_name );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   printf("unlink %s\n", Links_to_Dirs[5]);
   status = unlink( Links_to_dirlinks[5] );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   status = chdir( Directories[0] );
   status = mkdir ( "my_mount_point", S_IRWXU );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   printf("Attempting to mount IMFS file system at /dir1/my_mount_point \n");
   status = mount(
-     &mt_entry,
-     &IMFS_ops,
+    "null",
+    "/my_mount_point/dir1/my_mount_point",
+    "imfs",
      RTEMS_FILESYSTEM_READ_WRITE,
-     NULL,
-     "/my_mount_point/dir1/my_mount_point" );
-  assert( status == 0 );
+     NULL );
+  rtems_test_assert( status == 0 );
 
   printf("rmdir /dir1/my_mount_point should fail with EBUSY\n");
   status = rmdir ("/my_mount_point/dir1/my_mount_point" );
-  assert( status == -1 );
-  assert( errno == EBUSY );
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == EBUSY );
 
   printf( "Unmount /my_mount_point/dir1/my_mount_point\n");
   status = unmount( "/my_mount_point/dir1/my_mount_point" );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   /*
    * Verify write permission is checked.
@@ -674,30 +671,30 @@ void Cause_faults()
 
   printf("chmod of %s to group and other execute\n", Files[0] );
   status = chmod (Files[0], (S_IXGRP | S_IXOTH) );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
   printf("Open %s for write should fail with EACCES\n", Files[0] );
   fd = open (Files[0], O_WRONLY);
-  assert( fd == -1 );
-  assert( errno == EACCES );
+  rtems_test_assert( fd == -1 );
+  rtems_test_assert( errno == EACCES );
 
   printf("chmod of %s to User Execute and Read\n", Directories[3] );
   status = chmod (Directories[3], (S_IXUSR | S_IRUSR) );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
   strcpy(longer_name, Directories[3] );
   strcat(longer_name, "/NewFile" );
   printf("Mkdir of %s should fail with EACCES\n",longer_name );
   status = mkdir( longer_name, S_IRWXU );
-  assert( status != 0 );
-  assert( errno == EACCES );
+  rtems_test_assert( status != 0 );
+  rtems_test_assert( errno == EACCES );
 
   printf("Making too many hard links.\n" );
   make_too_many_links( );
 
   printf( "pass fstat a null pointer should fail with EFAULT\n");
   status = fstat( fd, NULL );
-  assert( status == -1 );
-  assert( errno == EFAULT);
+  rtems_test_assert( status == -1 );
+  rtems_test_assert( errno == EFAULT);
 
   /*
    * The current directory MUST be restored at the end of this test.
@@ -705,18 +702,18 @@ void Cause_faults()
 
   printf("chdir to /my_mount_point \n");
   status = chdir( "/my_mount_point" );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
 }
 
-void Show_Time()
+void Show_Time(void)
 {
   rtems_time_of_day time;
   rtems_status_code status;
 
-  status = rtems_clock_get( RTEMS_CLOCK_GET_TOD, &time );
-  printf(">>>>Current Time: ");
-  print_time( " - rtems_clock_get - ", &time, "\n" );
+  status = rtems_clock_get_tod( &time );
+  printf("--->Current Time: ");
+  print_time( " - rtems_clock_get_tod - ", &time, "\n" );
 }
 
 /*
@@ -732,9 +729,8 @@ int main(
 )
 #endif
 {
-  rtems_status_code                    status;
-  rtems_time_of_day                    time;
-  rtems_filesystem_mount_table_entry_t *mt_entry;
+  rtems_status_code status;
+  rtems_time_of_day time;
 
   puts( "\n\n*** STAT TEST 01 ***" );
 
@@ -749,18 +745,18 @@ int main(
    */
 
   status = mkdir("/my_mount_point",  S_IRWXU );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
   status = mount(
-     &mt_entry,
-     &IMFS_ops,
+    "null",
+    "my_mount_point",
+    "imfs",
      RTEMS_FILESYSTEM_READ_WRITE,
-     NULL,
-     "my_mount_point" );
-  assert( status == 0 );
+     NULL );
+  rtems_test_assert( status == 0 );
   status = chdir( "/my_mount_point" );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
   status = mkdir("dev",  S_IRWXU );
-  assert( status == 0 );
+  rtems_test_assert( status == 0 );
 
 
   /*
